import type {IBaseWorld} from './world.js';
import type {ICoord, IQuaternion, IVec3} from './threed.js';

export interface IPhysicsObject {
  position: ICoord;
  quaternion: IQuaternion;
  angularVelocity?: IVec3;
  velocity?: IVec3;
}

export class BasePhysics {
  baseWorld: IBaseWorld;
  bodies: {[id: string]: IPhysicsObject} = {};
  constructor(baseWorld: IBaseWorld) {
    this.baseWorld = baseWorld;
  }

  setPos(object: IPhysicsObject, body: IPhysicsObject) {
    object.position.x = body.position.x;
    object.position.y = body.position.y;
    object.position.z = body.position.z;
    object.quaternion.x = body.quaternion.x;
    object.quaternion.y = body.quaternion.y;
    object.quaternion.z = body.quaternion.z;
    object.quaternion.w = body.quaternion.w;
  }

  syncState() {
    for (const id in this.bodies) {
      const object = this.baseWorld.objects[id];
      const body = this.bodies[id];

      if (object && body) {
        this.setPos(object, body);
      }
    }
  }
}
