import * as CANNON from 'cannon-es';
import * as THREE from 'three';

export type TSceneObject = {
  id: string; object: THREE.Object3D;
  shape?: CANNON.Shape;
  size?: {x: number, y: number, z: number};
  mass?: number;
};

const light: TSceneObject = {
  id: 'light',
  object: new THREE.AmbientLight(0xFFFFFF, 0.9)
};

const floor: TSceneObject = {
  id: 'floor',
  object: function() {
    const floor = new THREE.Object3D();
    const mesh = new THREE.Mesh(
        new THREE.BoxGeometry(100, 10, 100, 1),
        new THREE.MeshPhongMaterial({color: 0x002288}));
    mesh.receiveShadow = true;
    floor.add(mesh);
    const grid = new THREE.GridHelper(100, 20, 0xFF00FF, 0x880088);
    grid.position.y = 5;
    floor.add(grid);
    return floor;
  }(),
  // shape: new CANNON.Box(new CANNON.Vec3(50, 0.05, 50)),
  size: {x: 100, y: 10, z: 100}
  // shape: new CANNON.Plane()
};

const floor2: TSceneObject = {
  id: 'floor2',
  object: function() {
    const floor2 = new THREE.Object3D();
    const mesh = new THREE.Mesh(
        new THREE.BoxGeometry(150, 10, 150, 1),
        new THREE.MeshPhongMaterial({color: 0x7500FF}));
    mesh.receiveShadow = true;
    floor2.add(mesh);
    const grid = new THREE.GridHelper(150, 20, 0xFF00FF, 0x880088);
    grid.position.y = 5;
    floor2.add(grid);
    floor2.position.set(-100, -20, 0);
    return floor2;
  }(),
  // shape: new CANNON.Box(new CANNON.Vec3(50, 0.05, 50)),
  size: {x: 150, y: 10, z: 150}
  // shape: new CANNON.Plane()
};

const planet: TSceneObject = {
  id: 'planet',
  object: function() {
    const planet = new THREE.Object3D();
    const mesh = new THREE.Mesh(
        new THREE.SphereGeometry(75),
        new THREE.MeshPhongMaterial({color: 0x38761D}));
    mesh.receiveShadow = true;
    planet.add(mesh);
    planet.position.set(200, -20, 0);
    return planet;
  }(),
  shape: new CANNON.Sphere(75),
};

const donut: TSceneObject = {
  id: 'donut',
  object: function() {
    const donut = new THREE.Object3D();
    const mesh = new THREE.Mesh(
        new THREE.RingGeometry(12.5, 75),
        new THREE.MeshPhongMaterial({color: 0x877652}));
    mesh.receiveShadow = true;
    mesh.rotation.x = -Math.PI / 2;
    donut.add(mesh);
    donut.position.set(0, -40, 200);
    return donut;
  }(),
  size: {x: 150, y: 10, z: 150}
};

export const TestScene: TSceneObject[] = [
  light,
  floor,
  floor2,
  planet,
  donut,
  ...function() {
    let i: number = 1;
    const boxes: TSceneObject[] = [];
    do {
      const box = new THREE.Object3D();
      const mesh = new THREE.Mesh(
          new THREE.BoxGeometry(1, 1, 1),
          new THREE.MeshPhongMaterial({color: Math.sin(i) * 0xFFFFFF}));
      mesh.receiveShadow = true;
      box.add(mesh);
      box.position.x = Math.sin(i) - 0.5;
      box.position.y = i * 10

      boxes.push({id: 'cube_' + i, object: box, mass: Math.random()});

      i++;
    } while (i <= 20)
    return boxes;
  }(),
  ...function() {
    let i: number = 1;
    const spheres: TSceneObject[] = [];
    do {
      const sphere = new THREE.Object3D();
      const mesh = new THREE.Mesh(
          new THREE.SphereGeometry(.5),
          new THREE.MeshPhongMaterial({color: Math.cos(i) * 0xFFFFFF}));
      mesh.receiveShadow = true;
      sphere.add(mesh);
      sphere.position.x = 5 + Math.sin(i) - 0.5;
      sphere.position.y = i * 10;

      spheres.push({
        id: 'sphere_' + i,
        object: sphere,
        shape: new CANNON.Sphere(.5),
        mass: Math.random()
      });

      i++;
    } while (i <= 20)
    return spheres;
  }(),
  ...function() {
    let i: number = 1;
    const objs: TSceneObject[] = [];
    do {
      const box = new THREE.Object3D();
      const mesh = new THREE.Mesh(
          new THREE.ConeGeometry(.5),
          new THREE.MeshPhongMaterial({color: Math.tan(i) * 0xFFFFFF}));
      mesh.receiveShadow = true;
      box.add(mesh);
      box.position.x = -5 + Math.sin(i) - 0.5;
      box.position.y = i * 10;

      objs.push({
        id: 'cone_' + i,
        object: box,
        shape: new CANNON.Cylinder(.01, .5, 1),
        mass: Math.random()
      });

      i++;
    } while (i <= 20)
    return objs;
  }(),
];