import type {IBaseTroonia} from '../common/troonia.js';
import type {Server} from 'socket.io';
import {World} from './world.js';
import {BaseTroonia} from '../common/troonia.js';
import type {Socket} from 'socket.io';
import {TestScene} from '../common/scenes.js';
// import type {Body} from 'cannon-es';
import type {IPlayerData} from '../common/player.js';

export class Troonia extends BaseTroonia implements IBaseTroonia {
  socket: Server;
  world: World;

  constructor(socket: Server) {
    super();
    this.socket = socket;

    this.socket.on('connection', (socket: any) => {
      this.onConnect(socket);
    });

    this.world = new World(this);
    this.world.load(TestScene);

    setInterval(() => {
      this.socket.emit('syncState', {objects: this.world.objects});
    }, 1000 / 20);

    setInterval(() => {
      this.update();
    }, 1000 / 50);

    // setInterval(() => {
    //   this.world.objects = {};
    //   for (const body of Object.values(this.world.physics.bodies)) {
    //     this.world.physics.world.removeBody(body as Body);
    //   }
    //   this.world.physics.bodies = {};
    //   this.world.load(TestScene);
    // }, 1000 * 10);
  }

  onConnect(socket: Socket) {
    console.log(`User ${socket.id} connected.`);

    socket.emit('syncPlayers', this.world.players);

    this.socket.emit('addPlayer', socket.id);
    this.world.addPlayer(socket.id);

    socket.on('disconnect', (message: any) => {
      this.onDisconnect(message, socket);
    });

    socket.on('updatePlayer', (data: IPlayerData) => {
      this.world.players[socket.id] = data;
    });
  }

  onDisconnect(message: any, socket: any) {
    this.world.removePlayer(socket.id);
    this.socket.emit('removePlayer', socket.id);
    console.log(`User ${socket.id} disconnected: ${message}.`);
  }

  update() {
    const delta = this.clock.getDelta();
    this.world.update(delta);
  }
}