import type {IBaseWorld} from '../common/world.js';
import type {IPhysicsObject} from '../common/physics.js'
import {Physics} from './physics.js';
import {Box3} from 'three';
import {BaseWorld} from '../common/world.js';
import type {Troonia} from './troonia.js';
import type {TSceneObject} from '../common/scenes.js';
import {Sphere} from 'cannon-es';
import type {IVec3} from '../common/threed';

export class World extends BaseWorld implements IBaseWorld {
  physics: Physics;

  constructor(game: Troonia) {
    super(game);
    this.physics = new Physics(this);
  }

  addPlayer(id: string) {
    const playerObject: IPhysicsObject = {
      position: {x: Math.random() * 90 - 45, y: 10, z: Math.random() * 90 - 45},
      quaternion: {x: 0, y: 0, z: 0, w: 1}
    };
    this.objects[id] = playerObject;
    this.physics.addObject(
        id, playerObject, 10, {x: 1, y: 1, z: 1}, new Sphere(.5));
    this.players[id] = {vec3: {x: 0, y: 0, z: 0}};
  }

  removePlayer(id: string) {
    delete this.players[id];
    delete this.objects[id];
    this.physics.removeObject(id);
  }

  update(delta: number) {
    this.physics.update(delta);
  }

  load(scene: TSceneObject[]) {
    for (const sceneObject of scene) {
      const physicsObject: IPhysicsObject = {
        position: {x: 0, y: 0, z: 0},
        quaternion: {x: 0, y: 0, z: 0, w: 0}
      };

      physicsObject.position = {
        x: sceneObject.object.position.x,
        y: sceneObject.object.position.y,
        z: sceneObject.object.position.z
      };

      physicsObject.quaternion = {
        x: sceneObject.object.quaternion.x,
        y: sceneObject.object.quaternion.y,
        z: sceneObject.object.quaternion.z,
        w: sceneObject.object.quaternion.w
      };

      let size: IVec3;
      if (!sceneObject.size) {
        size = function() {
          const bounds = new Box3().setFromObject(sceneObject.object);
          return {
            x: Math.abs(bounds.max.x - bounds.min.x),
            y: Math.abs(bounds.max.y - bounds.min.y),
            z: Math.abs(bounds.max.y - bounds.min.y)
          };
        }();
      } else {
        size = sceneObject.size;
      }

      let mass = ((sceneObject.mass) ? sceneObject.mass : 0);

      this.objects[sceneObject.id] = physicsObject;
      this.physics.addObject(
          sceneObject.id, physicsObject, mass, size, sceneObject.shape);
    }
  }
}